The following options can be passed to any command:

<%= table(options.map{ |option| [option[:switches].join(', '), option[:description]] }, table_args('  ', 26)).join("\n") %>

The following configuration options are stored in the <%= RHC::Helpers.system_path('.openshift/express.conf') %> file in your home directory.  Passing any of these options to 'rhc setup' will save that value to the file:

<%= table(
	  RHC::Config::OPTIONS.select{ |key,value| not value.nil? }.map do |key,value| 
		["--#{key.to_s.gsub('_','-')}", value[0] || key]
	  end, 
	  table_args('  ', 26).merge(:header => ['Option', 'In Config'])
	).join("\n") -%>
